from .columns import TableColumnBronze, TableColumnSilver
from datetime import datetime


class DataTableBronze:
    def __init__(self, schema_name, table_prefixo, table_name, columns=[]):
        self.columns = columns
        self.schema_name = schema_name
        self.table_name = table_name
        self.table_prefixo = table_prefixo

    def add_column(
        self, name: str, data_type: str, nullable: bool, column_type: str = ""
    ):
        column = TableColumnBronze(name, "", data_type, nullable, column_type)
        self.columns.append(column)

    def getPk(self):
        for column in self.columns:
            if column.column_type.upper() == "PK":
                return column
        return None

    def getNoPk(self):
        non_pk_columns = []
        for column in self.columns:
            if column.column_type.upper() != "PK":
                non_pk_columns.append(column)
        return non_pk_columns


class DataTableSilver:
    def __init__(self, schema_name, table_prefixo, table_name, columns=[]):
        self.schema_name = schema_name
        self.table_prefixo = table_prefixo
        self.table_name = table_name
        self.columns = columns
        setattr(self, f"{self.table_prefixo}_dh_carga", datetime.now())

    def add_column(
        self, name: str, data_type: str, nullable: bool, column_type: str = ""
    ):
        column = TableColumnSilver(name, data_type, nullable, column_type)
        self.columns.append(column)

    def getPk(self):
        for column in self.columns:
            if column.column_type == "PK":
                return column
        return None


class DataTableSilverHub(DataTableSilver):
    def __init__(self, schema_name, table_prefixo, table_name, columns=[]):
        super().__init__(schema_name, table_prefixo, table_name, columns)


class DataTableSilverLink(DataTableSilver):
    def __init__(self, schema_name, table_prefixo, table_name, columns=[]):
        super().__init__(schema_name, table_prefixo, table_name, columns)


class DataTableSilverSatellite(DataTableSilver):
    def __init__(self, schema_name, table_prefixo, table_name, columns=[]):
        super().__init__(schema_name, table_prefixo, table_name, columns)
        setattr(self, f"{self.table_prefixo}_dh_expiracao", None)
        setattr(self, f"{self.table_prefixo}_st_deletado", None)



from typing import List
from datatable.tables import DataTableBronze, DataTableSilver


class Bronze_to_silver:
    def __init__(
        self,
        spark,
        datatable_bronze: DataTableBronze,
        datatable_silver: DataTableSilver,
    ):
        self.spark = spark
        self.datatable_bronze = datatable_bronze
        self.datatable_silver = datatable_silver

    def create_database_if_not_exists(self):
        self.spark.sql(
            f"CREATE DATABASE IF NOT EXISTS {self.datatable_silver.table_name}"
        )

    def create_table_silver_if_not_exists(self, manager_type="iceberg"):
        columns_sql = ", ".join(
            [str(column) for column in self.datatable_silver.columns]
        )
        sql = f"""
            CREATE TABLE IF NOT EXISTS {self.datatable_silver.schema_name}.{self.datatable_silver.table_name} (
                {columns_sql}
            )
            USING {manager_type}
        """
        self.spark.sql(sql)

    # todo
    def build_comparison_query(self):
        return NotImplementedError(
            "TODO:No futuro essa classe fará o build da query de compraração"
        )

    def merge_data(self, comparison_query):
        sql = f"""
            MERGE INTO {self.datatable_silver.schema_name}.{self.datatable_silver.table_name} AS t
            USING ({comparison_query}) AS s
            ON t.{self.datatable_silver.getPk().name} = s.{self.datatable_bronze.getPk().name}
            WHEN MATCHED THEN
                UPDATE SET t.{self.datatable_silver.table_prefixo}_dh_carga = s.{self.datatable_silver.table_prefixo}_dh_carga
            WHEN NOT MATCHED THEN
                INSERT *
        """
        self.spark.sql(sql)

    def create_or_replace(self, comparison_query):
        self.create_database_if_not_exists()
        self.create_table_silver_if_not_exists()
        self.merge_data(comparison_query)



class TableColumnBronze:
    def __init__(
        self,
        name: str,
        alias: str,
        data_type: str,
        nullable: bool,
        column_type: str = "",
    ):
        self.name = name
        self.alias = alias
        self.data_type = data_type
        self.nullable = nullable
        self.column_type = column_type

    def __str__(self):
        nullable_str = "NOT NULL" if not self.nullable else ""
        return f"{self.name} {self.data_type} {nullable_str}".strip()


class TableColumnSilver:
    def __init__(
        self, name: str, data_type: str, nullable: bool, column_type: str = ""
    ):
        self.name = name
        self.data_type = data_type
        self.nullable = nullable
        self.column_type = column_type

    def __str__(self):
        nullable_str = "NOT NULL" if not self.nullable else ""
        return f"{self.name} {self.data_type} {nullable_str}".strip()
